import hashlib
import os
import sys
import zipfile
from enum import Enum

from requests import exceptions

#from gdelttools.mongoimport import MongoImport
from gdelttools.web import WebDownload


def compute_md5(file):
    hasher = hashlib.md5()
    with open(file, 'rb') as input:
        buf = input.read()
        hasher.update(buf)

    return hasher.hexdigest()


class GDELTChecksumError(ValueError):
    pass


class GDELTZipError(ValueError):
    pass


class GDELTFilter(Enum):

    all = "all"
    gkg = "gkg"
    mentions = "mentions"
    export = "export"

    def __str__(self):
        return self.value


class GDELTFile:
    TEST_FILES = """150383 297a16b493de7cf6ca809a7cc31d0b93 http://data.gdeltproject.org/gdeltv2/20150218230000.export.CSV.zip
318084 bb27f78ba45f69a17ea6ed7755e9f8ff http://data.gdeltproject.org/gdeltv2/20150218230000.mentions.CSV.zip
10768507 ea8dde0beb0ba98810a92db068c0ce99 http://data.gdeltproject.org/gdeltv2/20150218230000.gkg.csv.zip
"""

    def __init__(self, url:str, size:int, md5:str, gfilter: GDELTFilter = GDELTFilter.all):
        self._url = url
        self._size = size
        self._md5 = md5
        filename_with_args = url.split('/')[-1]
        self._zip_filename = filename_with_args.split('?')[0]
        self._csv_filename = os.path.splitext(self._zip_filename)[0]
        self._wd = WebDownload()
        self._filter = gfilter

    @property
    def filter(self):
        return self._filter

    @filter.setter
    def filter(self, rhs:GDELTFilter):
        self._filter = rhs

    @property
    def url(self):
        return self._url

    @property
    def size(self):
        return self._size

    @property
    def md5(self):
        return self._md5

    @property
    def zip_filename(self):
        return self._zip_filename

    @property
    def csv_filename(self):
        return self._csv_filename

    def download_file(self):
        print(f"{self.url} --> ", end="")
        self._zip_filename = self._wd.download_url(self.url)
        if not self.is_valid_checksum():
            raise GDELTChecksumError(self.url)
        print(f"{self.zip_filename}")
        return self.zip_filename

    def is_valid_checksum(self):
        computed_md5 = compute_md5(self._zip_filename)
        return computed_md5 == self.md5

    def process_zip_file(self, overwrite: bool = True):
        #
        # If overwrite download and extract
        # else
        if overwrite:
            self._zip_filename = self.download_file()
            self._csv_filename = self.extract_csv_file()
            return self.csv_filename

        elif os.path.exists(self.csv_filename):
            print(f"{self.csv_filename} exists")
        elif os.path.exists(self.zip_filename):
            print(f"{self.zip_filename} exists")
            csv_filename = self.extract_csv_file()
        else:
            self._zip_filename = self.download_file()

            self._csv_filename = self.extract_csv_file()
        return self._csv_filename

    @staticmethod
    def unzip(filename: str):
        zfilename = None
        with zipfile.ZipFile(filename) as archive:
            if len(archive.namelist()) > 1:
                raise GDELTZipError(f"More than one file in archive: {filename}")
            for zfilename in archive.namelist():
                text = archive.read(zfilename)  # .decode(encoding="utf-8")
                print(f"extracting: '{zfilename}'")
                with open(zfilename, "wb") as output_file:
                    output_file.write(text)
        return zfilename

    def extract_csv_file(self):
        self._csv_filename = GDELTFile.unzip(self.zip_filename)
        return self._csv_filename

    @classmethod
    def get_input_files(cls, f: str, last : int):
        lines = []
        with open(f, "r") as input_file:
            for input_line in input_file:
                lines.append(input_line)
            if last > 0:
                section = len(lines) - (last * 3)  # three files per set, gkg, export, mentions
                lines = lines[section:]  # slice the last days
        return lines


def download_gdelt_files(file_list: list[str], last=None, filter:GDELTFilter=GDELTFilter.all,  overwrite=False):

    if last is None or last < 0:
        last = 0

    csv_files: list[str] = []
    for f in file_list:
        lines = GDELTFile.get_input_files(f, last)
        # with open(f, "r") as input_file:
        #     for input_line in input_file:
        #         lines.append(input_line)
        #
        #     if last > 0:
        #         section = len(lines) - (last * 3)  # three files per set, gkg, exports, mentions
        #         lines = lines[section:]  # slice the last days

        #importer = MongoImport()
        for l in lines:
            try:
                size, md5, zipurl = l.split()
                gdelt_file = GDELTFile(zipurl, int(size), md5, filter)
                # print(f"{size}:{sha}:{zip}")
                if (gdelt_file.filter.value in zipurl) or (gdelt_file.filter == GDELTFilter.all):
                    f = gdelt_file.process_zip_file(overwrite)
                    #importer.command(f)
                    csv_files.append(f)

            except zipfile.BadZipfile as e:
                print(gdelt_file.zip_filename)
                print(e)
                sys.exit(1)
            except exceptions.HTTPError as e:
                print(f"Error for {zipurl}")
                print(e)
                sys.exit(1)
            except GDELTChecksumError as e:
                print(f"'{gdelt_file.md5}' checksum for {gdelt_file.url} doesn't match\n"
                      f" checksum for {gdelt_file.zip_filename}")
                sys.exit(1)
    return csv_files



import argparse
import pymongo
import pprint

def main():

    parser = argparse.ArgumentParser()

    parser.add_argument("--host",
                        default="http://localhost:27017",
                        help="MongoDB URI")

    parser.add_argument("--sample", type=int, default=10,
                        help = "The sample size : default to 10")

    args = parser.parse_args()

    client = pymongo.MongoClient(args.host)

    db = client["GDELT2"]
    source = db["eventscsv"]
    sink = db["events"]

    sample_cursor = source.aggregate([{"$sample" : {"size": 10}}])

    for d in sample_cursor:
        pprint.pprint(d)





import os
from enum import Enum
import shutil
import glob

class BinaryNotFoundError(OSError):
    pass

class FileType(Enum):
    tsv = "tsv"
    csv = "csv"

    def __str__(self):
        return self.value


class InsertMode(Enum):
    upsert = "upsert"

    def __str__(self):
        return self.value




class MongoImport:
    '''
    Implement this script for mongoimport. We expect mongoimport to be installed
    and be available on the path.

    mongoimport --db=GDELT2 --collection=eventscsv  --type=tsv --fieldFile=gdelt_field_file.ff --mode=upsert --writeConcern "{w:1}" --columnsHaveTypes --file=$i $*
    '''

    def __init__(self,
                 prog: str = "mongoimport",
                 uri: str = "mongodb://localhost:27017",
                 database_name: str="GDELT2",
                 collection_name: str="eventscsv",
                 file_type : FileType = FileType.tsv,
                 field_file: str = "gdelt_field_file.ff",
                 insert_mode : InsertMode = InsertMode.upsert,
                 write_concern : str = "{w:1}"):
        """
        Define init parameters for mongoimport command
        :param database: The mongodb database name we will be importing to
        :param collection: The collection name we will be importing to
        """

        self._prog = prog
        self._uri = uri
        self._database_name = database_name
        self._collection_name = collection_name
        self._file_type = file_type
        self._field_file = field_file
        self._insert_mode = insert_mode
        self._write_concern = write_concern

        if shutil.which(self._prog) is None:
            raise BinaryNotFoundError(f"{self._prog}: Cannot be found on the current PATH")

    def command(self, input_file:str):

        return f"mongoimport --uri {self._uri} --db {self._database_name} --collection {self._collection_name} " +\
               f"--type {self._file_type.value} --fieldFile {self._field_file} " +\
               f"--mode {self._insert_mode.value} --writeConcern '{self._write_concern}' --columnsHaveTypes " +\
               f"--file {input_file}"

    def import_data(self, arg:str):
        args = glob.glob(arg)
        for i in args:
            print(f"Running in: {os.getcwd()}")
            print(self.command(i))
            os.system(f"{self.command(i)}")


if __name__ == "__main__":
    m = MongoImport()
    m.import_data("*.export.CSV")




import argparse

import pymongo

if __name__ == "__main__":
    parser = argparse.ArgumentParser()

    parser.add_argument("--host", default="mongodb://localhost:27017",
                        help="MongoDB URI [%(default)s]")

    parser.add_argument("--database", default="GDELT",
                        help="Default database for loading [%(default)s]")

    parser.add_argument("-i", "--inputcollection", default="events_csv",
                        help="Default collection for input [%(default)s]")
    parser.add_argument("-o", "--outputcollection", default="events",
                        help="Default collection for output [%(default)s]")

    args = parser.parse_args()

    client = pymongo.MongoClient(host=args.host)
    db = client[args.database]
    input_collection = db[args.inputcollection]
    output_collection = db[args.outputcollection]

    matcher = {"$match": {"ActionGeo_Lat": {"$type": "double"},
                          "ActionGeo_Long": {"$type": "double"},
                          "Actor1Geo_Lat": {"$type": "double"},
                          "Actor1Geo_Long": {"$type": "double"},
                          "Actor2Geo_Lat": {"$type": "double"},
                          "Actor2Geo_Long": {"$type": "double"}}}

    adder = {"$addFields": {"Actor1Geo": {"type": "Point", "coordinates": ["$Actor1Geo_Long", "$Actor1Geo_Lat"]},
                            "Actor2Geo": {"type": "Point", "coordinates": ["$Actor2Geo_Long", "$Actor2Geo_Lat"]},
                            "ActionGeo": {"type": "Point", "coordinates": ["$ActionGeo_Long", "$ActionGeo_Lat"]}}}

    deleter = {"$unset": ["ActionGeo_Lat", "ActionGeo_Long", "Actor1Geo_Lat",
                          "Actor1Geo_Long", "Actor2Geo_Lat", "Actor2Geo_Long"]}

    merger = {"$merge": {"into": args.outputcollection,
                         "on": "_id",
                         "whenMatched": "replace",
                         "whenNotMatched": "insert"}}
    input_collection.aggregate([matcher, adder, deleter, merger])
    output_collection = db[args.outputcollection]
    print(f"Processed documents total : {output_collection.count_documents({})}")



from datetime import datetime
from gdelttools.web import WebDownload
import requests


class GDELTWebData:

    gdelt_md5_list = "http://data.gdeltproject.org/events/md5sums"
    gdelt_file_sizes = "http://data.gdeltproject.org/events/filesizes"
    checksum_filename = "gdelt_md5sums"
    size_filename = "gdelt_filesizes"

    master_url = "http://data.gdeltproject.org/gdeltv2/masterfilelist.txt"
    update_url = "http://data.gdeltproject.org/gdeltv2/lastupdate.txt"

    downloader = WebDownload()

    @classmethod
    def get_metadata(cls):
        # size and MD5 checksums to validate downloads
        r = requests.get(cls.gdelt_md5_list, allow_redirects=True)
        with open(cls.checksum_filename, "w") as output_file:
            output_file.write(r.content.decode("utf-8"))
        r = requests.get(cls.gdelt_file_sizes, allow_redirects=True)
        with open(cls.size_filename, "w") as output_file:
            output_file.write(r.content.decode("utf-8"))
        return [cls.checksum_filename, cls.size_filename]

    @classmethod
    def get_update_list(cls, overwrite=False):
        return cls.downloader.download_url(cls.update_url)

    @classmethod
    def get_master_list(cls, overwrite=False):
        return cls.downloader.download_url(cls.master_url)




import unittest
import os
import zipfile

from gdelttools.gdeltfile import GDELTFile, GDELTFilter, download_gdelt_files


class TestGDeltFile(unittest.TestCase):
    size = 150383
    md5 = "297a16b493de7cf6ca809a7cc31d0b93"
    url = "http://data.gdeltproject.org/gdeltv2/20150218230000.export.CSV.zip"

    def test_gdelt_file(self):
        f = GDELTFile(self.url, self.size, self.md5, GDELTFilter.all)
        self.assertEqual(self.url, f.url)
        self.assertEqual(self.size, f.size)
        self.assertEqual(self.md5, f.md5)
        self.assertEqual(GDELTFilter.all, f.filter)
        f = GDELTFile(self.url, self.size, self.md5, GDELTFilter.gkg)
        self.assertEqual(GDELTFilter.gkg, f.filter)
        self.assertEqual(GDELTFilter.gkg.value, "gkg")
        self.assertTrue(GDELTFilter.export.value in self.url)

    def test_download(self):
        f = GDELTFile(self.url, self.size, self.md5, GDELTFilter.all)
        f.download_file()
        self.assertTrue(os.path.isfile(f.zip_filename))
        os.unlink(f.zip_filename)

    def test_extract(self):
        f = GDELTFile(self.url, self.size, self.md5, GDELTFilter.all)
        f.download_file()
        self.assertTrue(os.path.isfile(f.zip_filename))
        f.extract_csv_file()
        self.assertTrue(os.path.isfile(f.csv_filename))
        os.unlink(f.zip_filename)
        os.unlink(f.csv_filename)

    def test_download_gdelt_files(self):

        with open("threefiles.txt", "w") as tfile:
            tfile.write(GDELTFile.TEST_FILES)

        download_gdelt_files(["threefiles.txt"], overwrite=True)
        self.assertTrue(os.path.exists("20150218230000.export.CSV.zip"))
        self.assertTrue(os.path.exists("20150218230000.mentions.CSV.zip"))
        self.assertTrue(os.path.exists("20150218230000.gkg.csv.zip"))
        os.unlink("20150218230000.export.CSV.zip")
        os.unlink("20150218230000.mentions.CSV.zip")
        os.unlink("20150218230000.gkg.csv.zip")
        os.unlink("20150218230000.export.CSV")
        os.unlink("20150218230000.mentions.CSV")
        os.unlink("20150218230000.gkg.csv")

        download_gdelt_files(["threefiles.txt"])
        self.assertTrue(os.path.exists("20150218230000.export.CSV.zip"))
        self.assertTrue(os.path.exists("20150218230000.mentions.CSV.zip"))
        self.assertTrue(os.path.exists("20150218230000.gkg.CSV.zip"))
        os.unlink("20150218230000.export.CSV.zip")
        os.unlink("20150218230000.mentions.CSV.zip")
        os.unlink("20150218230000.gkg.csv.zip")
        os.unlink("20150218230000.export.CSV")
        os.unlink("20150218230000.mentions.CSV")
        os.unlink("20150218230000.gkg.csv")

        download_gdelt_files(["threefiles.txt"], last=0, overwrite=True, filter=GDELTFilter.gkg)
        self.assertTrue(os.path.exists("20150218230000.gkg.csv.zip"))
        self.assertFalse(os.path.exists("20150218230000.export.CSV.zip"))
        self.assertFalse(os.path.exists("20150218230000.mentions.CSV.zip"))
        os.unlink("20150218230000.gkg.csv.zip")
        os.unlink("20150218230000.gkg.csv")

        download_gdelt_files(["threefiles.txt"], last=0, filter=GDELTFilter.mentions)
        self.assertFalse(os.path.exists("20150218230000.gkg.csv.zip"))
        self.assertFalse(os.path.exists("20150218230000.export.CSV.zip"))
        self.assertTrue(os.path.exists("20150218230000.mentions.CSV.zip"))
        os.unlink("20150218230000.mentions.CSV.zip")
        os.unlink("20150218230000.mentions.CSV")

        with open("threefiles.zip", "w") as tfile:
            tfile.write(GDELTFile.TEST_FILES)

        self.assertRaises(zipfile.BadZipfile, GDELTFile.unzip, "threefiles.zip")
        os.unlink("threefiles.zip")
        os.unlink("threefiles.txt")


if __name__ == '__main__':
    unittest.main()



import unittest
import os
from gdelttools.web import WebDownload, local_path


class TestDownload(unittest.TestCase):

    photo_name = "london.jpg"
    photo_url= "https://jdrumgoole.files.wordpress.com/2018/10/20180927_173519.jpg"
    text_filename = "testmasterfilelist.txt"
    text_url = "http://joedrumgoole.com/testmasterfilelist.txt"

    def setUp(self):
        self._wd = WebDownload()

    def test_local_path(self):
        p = local_path(self.photo_url)
        self.assertEqual(p, "20180927_173519.jpg")

    def test_download_chunks(self):
        if os.path.exists(self.photo_name):
            os.unlink(self.photo_name)
        self._wd.download_url(self.photo_url, self.photo_name)
        self.assertTrue(os.path.exists(self.photo_name))
        os.unlink(self.photo_name)

    def test_download_lines(self):
        if os.path.exists(self.text_filename):
            os.unlink(self.text_filename)
        with open(self.text_filename, "w") as output_file:
            for i,line in enumerate(self._wd.download_lines(self.text_url), 1):
                output_file.write(f"{line}")
            self.assertEqual(i, 2000)
        self.assertEqual(os.path.getsize(self.text_filename), 213940)
        os.unlink(self.text_filename)

    def test_download_url(self):
        self._wd.download_url(self.text_url, self.text_filename+"1")
        url_size = os.path.getsize(self.text_filename+"1")
        self.test_download_lines()
        line_size = os.path.getsize(self.text_filename+"1")
        self.assertEqual(url_size, line_size)
        os.unlink(self.text_filename+"1")

if __name__ == '__main__':
    unittest.main()




import urllib
from io import BytesIO
import os

import zipfile
import requests


def download_and_unzip(u: str):
    url_file = urllib.request.urlopen(u)

    with zipfile.ZipFile(BytesIO(url_file.read())) as my_zip_file:
        my_zip_file.extractall()


def skip_this_file(zip_filename: str, overwrite: bool = None):
    name, ext = os.path.splitext(zip_filename)
    csv_filename = f"{name}.CSV"
    if overwrite:
        return overwrite
    else:
        name, ext = os.path.splitext(zip_filename)
        csv_filename = f"{name}.CSV"
        return os.path.exists(zip_filename) or os.path.exists(csv_filename)


def local_path(url):
    filename_with_args = url.split('/')[-1]
    filename = filename_with_args.split('?')[0]
    return filename


class WebDownload:

    CHUNK_SIZE = 1024 * 1024

    def __init__(self, encoding = "utf-8", chunksize=None):
        self._encoding = encoding
        if chunksize:
            self._chunksize = chunksize
        else:
            self._chunksize = self.CHUNK_SIZE

    def download_chunks(self, url):
        filename = local_path(url)
        # NOTE the stream=True parameter below
        with requests.get(url, stream=True) as r:
            r.raise_for_status()
            for chunk in r.iter_content(chunk_size=self._chunksize):
                if chunk:
                    yield chunk

    def download_lines(self, url):
        filename = local_path(url)
        # NOTE the stream=True parameter below
        r = requests.get(url, stream=True)

        for line in r.iter_lines():
            if line:
                yield f"{line.decode(self._encoding)}\n"

    def download_url(self, url, target_filename=None):
        if target_filename is None:
            filename = local_path(url)
        else:
            filename = target_filename
        with open(filename, 'wb') as f:
            for i, chunk in enumerate(self.download_chunks(url), 1):
                f.write(chunk)
        return filename







import unittest

from gdelttools.mongoimport import MongoImport, BinaryNotFoundError


class TestMongoImport(unittest.TestCase):
    def test_mongoimport(self):

        with self.assertRaises(BinaryNotFoundError):
            x = MongoImport(prog="notmongoimport")  # add assertion here


if __name__ == '__main__':
    unittest.main()



"""
Importer for GDELT 2.0 raw data set.
https://blog.gdeltproject.org/gdelt-2-0-our-global-world-in-realtime/

The master file list is here:
http://data.gdeltproject.org/gdeltv2/masterfilelist.txt

"""
import argparse
import sys
import os
import pymongo

from gdelttools.gdeltfile import download_gdelt_files, GDELTFilter
from gdelttools.gdeltwebdata import GDELTWebData
from gdelttools._version import __version__
from gdelttools.mongoimport import MongoImport


def main():

    parser = argparse.ArgumentParser(epilog=f"Version: {__version__}\n"
                                            f"More info : https://github.com/jdrumgoole/gdelttools ")

    parser.add_argument("--host",
                        help="MongoDB URI")

    parser.add_argument("--database", default="GDELT2",
                        help="Default database for loading [%(default)s]")

    parser.add_argument("--collection", default="eventscsv",
                        help="Default collection for loading [%(default)s]")
    parser.add_argument("--master",
                        default=False,
                        action="store_true",
                        help="GDELT master file [%(default)s]")

    parser.add_argument("--update",
                        default=False,
                        action="store_true",
                        help="GDELT update file [%(default)s]")

    parser.add_argument("--local", type=str,
                        help="load data from local list of zips")

    parser.add_argument("--overwrite",
                        default=False, action="store_true",
                        help="Overwrite files when they exist already")

    parser.add_argument("--download", default=False, action="store_true",
                        help="download zip files from master or local file")

    parser.add_argument("--importdata", default=False, action="store_true",
                        help="Import files into MongoDB")
    parser.add_argument("--metadata", action="store_true", default=False,
                        help="grab meta data files")

    parser.add_argument("--filter", default="all", type=GDELTFilter, choices=list(GDELTFilter),
                        help="download a subset of the data, the default is all data [export, mentions gkg, all]")

    parser.add_argument("--last", default=0, type=int, help="how many recent files to download default : [%(default)s] implies all files")

    parser.add_argument('--version', action='version', version=f'%(prog)s {__version__}')
    args = parser.parse_args()

    # if args.ziplist == "master":
    #     url = args.master
    # else:
    #     url = args.incremental



    try:
        # files_collection = None

        # if args.host:
        #     client = pymongo.MongoClient(host=args.host)
        #     db = client[args.database]
        #     files_collection = db["files"]
        #     events_collection = db[args.collection]

        if args.metadata:
            GDELTWebData.get_metadata()

        input_file_list = []

        if args.master:
            print(f"{GDELTWebData.master_url} ", end="")
            filename = GDELTWebData.get_master_list()
            print(f"-> {filename}")
            input_file_list.append(filename)

        if args.update:
            print(f"{GDELTWebData.update_url} ", end="")
            filename = GDELTWebData.get_update_list()
            print(f"-> {filename}")
            input_file_list.append(filename)

        if args.local:
            if os.path.isfile(args.local):
                input_file_list.append(args.local)
            else:
                print(f"'{args.local}' does not exist")
                sys.exit(1)

        if args.download:
            if len(input_file_list) > 0:
                csv_files = download_gdelt_files(input_file_list, args.last, args.filter, args.overwrite)
            else:
                print(f"No files listed for download")

        if args.importdata:
            importer = MongoImport(database_name=args.database, collection_name=args.collection)
            for f in csv_files:
                importer.import_data(f)

    except KeyboardInterrupt:
        print("Exiting...")
        sys.exit(0)


if __name__ == "__main__":
    main()
